//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "Coinc2.h"

ClassImp(Coinc2)

////////////////////////////////////////////////////////////////////////////////////
Coinc2::Coinc2(const int aVerbose): GwollumPlot("coinc2","STANDARD"){
////////////////////////////////////////////////////////////////////////////////////

  // inputs
  verbosity=aVerbose;

  // init
  coinc_dt   = 0.0;
  coinc_df   = 0.0;
  coinc_af   = 1.0;
  CoTree     = new TTree("coinc","coinc");
  CoSeg      = new Segments();
  CoTag      = new bool [0];
   
  for(int i=0; i<2; i++){
    triggers[i]=NULL;
    toffset[i]=0.0;
    nactive[i]=0.0;
    freqmin[i] = -1.0;
    freqmax[i] = 1e20;
    h_ct[i]= new TH2D();
    h_ctcum[i]= new TH2D();
    h_ct_coinc[i]= new TH2D();
    h_ctcum_coinc[i]= new TH2D();
  }
  hc_asnr   = new TH2D();
  hc_snrsnr = new TH2D();
  hc_af     = new TH2D();
  hc_ff     = new TH2D();

}

////////////////////////////////////////////////////////////////////////////////////
Coinc2::Coinc2(ReadTriggers *aTriggerSample0, ReadTriggers *aTriggerSample1, const int aVerbose): GwollumPlot("coinc2","STANDARD"){
////////////////////////////////////////////////////////////////////////////////////

  // inputs
  verbosity=aVerbose;

  // init
  coinc_dt   = 0.0;
  coinc_df   = 0.0;
  coinc_af   = 1.0;
  CoTree     = new TTree("coinc","coinc");
  CoSeg      = new Segments();
  CoTag      = new bool [0];
  
  for(int i=0; i<2; i++){
    triggers[i]=NULL;
    toffset[i]=0.0;
    nactive[i]=0.0;
    freqmin[i] = -1.0;
    freqmax[i] = 1e20;
    h_ct[i]= new TH2D();
    h_ctcum[i]= new TH2D();
    h_ct_coinc[i]= new TH2D();
    h_ctcum_coinc[i]= new TH2D();
  }
  hc_asnr   = new TH2D();
  hc_snrsnr = new TH2D();
  hc_af     = new TH2D();
  hc_ff     = new TH2D();

  // set triggers
  SetTriggerSamples(aTriggerSample0, aTriggerSample1);
}

////////////////////////////////////////////////////////////////////////////////////
Coinc2::~Coinc2(void){
////////////////////////////////////////////////////////////////////////////////////
  if(verbosity>1) cout<<"Coinc2::~Coinc2"<<endl;
  delete CoTree;
  delete CoSeg;
  delete CoTag;
  for(int i=0; i<2; i++){
    CoC[i].clear();
    delete h_ct[i];
    delete h_ctcum[i];
    delete h_ct_coinc[i];
    delete h_ctcum_coinc[i];
  }
  delete hc_asnr;
  delete hc_snrsnr;
  delete hc_af;
  delete hc_ff;
}

////////////////////////////////////////////////////////////////////////////////////
bool Coinc2::SetTriggerSamples(ReadTriggers *aTriggerSample0, ReadTriggers *aTriggerSample1, const double aTimeOffset0, const double aTimeOffset1, Segments *aValidSegments, const bool aIgnoreMinusOnes){
////////////////////////////////////////////////////////////////////////////////////
  if(aTriggerSample0==NULL || !aTriggerSample0->GetStatus()){
    cerr<<"Coinc2::SetTriggerSamples: the trigger sample 0 is corrupted"<<endl;
    return false;
  }
  if(aTriggerSample1==NULL || !aTriggerSample1->GetStatus()){
    cerr<<"Coinc2::SetTriggerSamples: the trigger sample 1 is corrupted"<<endl;
    return false;
  }
  if(aValidSegments!=NULL&&!aValidSegments->GetStatus()){
    cerr<<"Coinc2::SetTriggerSamples: the input Segments object are corrupted"<<endl;
    return false;
  }
  if(verbosity>1) cout<<"Coinc2::SetTriggerSamples: sets new input trigger samples"<<endl;

  // link trigger objects
  triggers[0]=aTriggerSample0;
  triggers[1]=aTriggerSample1;
  toffset[0]=aTimeOffset0;
  toffset[1]=aTimeOffset1;

  // reset coinc segments
  delete CoSeg;

  // ************ MAKE COINC SEGMENTS ************
  // STEP1: take the first trigger sample segments
  CoSeg = new Segments(triggers[0]->GetSegments()->GetStarts(),triggers[0]->GetSegments()->GetEnds());

  // STEP2: restrict to valid segments (if any)
  if(aValidSegments!=NULL) CoSeg->Intersect(aValidSegments);

  // STEP3: apply time offset
  CoSeg->ApplyPadding(toffset[0],toffset[0]);

  // STEP4: take the second trigger sample segments
  Segments *tmp = new Segments(triggers[1]->GetSegments()->GetStarts(),triggers[1]->GetSegments()->GetEnds());

  // STEP5: restrict to valid segments (if any)
  if(aValidSegments!=NULL) tmp->Intersect(aValidSegments);

  // STEP6: apply time offset
  tmp->ApplyPadding(toffset[1],toffset[1]);
  
  // STEP6: intersect both segments
  CoSeg->Intersect(tmp);
  delete tmp;
  if(verbosity>1) cout<<"Coinc2::SetTriggerSamples: coincident live-time = "<<CoSeg->GetLiveTime()<<" s"<<endl;
  //***********************************************

  // reset coincs
  CoTree->Reset();
  delete CoTag;
  CoTag = new bool [0];
  CoC[0].clear();
  CoC[1].clear();
  
  // tag clusters based on coinc segments --> active clusters
  SelectClusters(0,aIgnoreMinusOnes);
  SelectClusters(1,aIgnoreMinusOnes);

  return true;
}

////////////////////////////////////////////////////////////////////////////////////
int Coinc2::MakeCoinc(void){
////////////////////////////////////////////////////////////////////////////////////
  if(triggers[0]==NULL || triggers[1]==NULL){
    cerr<<"Coinc2::MakeCoinc: both trigger samples must be defined"<<endl;
    return -1;
  }

  // reset coinc
  CoTree->Reset();
  for(int i=0; i<2; i++) CoC[i].clear();
  
  // locals
  double ts0, te0, ts1, te1;
  double f0, fs0, fe0, f1, fs1, fe1;
  int c1start=0;
  if(verbosity) cout<<"Coinc2::MakeCoinc: run the reference coincidence algo..."<<endl;

  // loop over trigger sample #0
  for(int c0=0; c0<triggers[0]->GetNClusters(); c0++){
    if(triggers[0]->GetClusterTag(c0)==-1) continue;
    triggers[0]->SetClusterTag(c0,1);
    ts0=triggers[0]->GetClusterTimeStart(c0)+toffset[0];
    te0=triggers[0]->GetClusterTimeEnd(c0)+toffset[0];
      
    // loop over trigger sample #1
    for(int c1=c1start; c1<triggers[1]->GetNClusters(); c1++){
      if(c1) c1start=c1-1;// assumes clusters with no time overlap
      if(triggers[1]->GetClusterTag(c1)==-1) continue;
      triggers[1]->SetClusterTag(c1,1);
 
      // time coincidence
      te1=triggers[1]->GetClusterTimeEnd(c1)+toffset[1];
      if(te1<=ts0-coinc_dt) continue;// before
      ts1=triggers[1]->GetClusterTimeStart(c1)+toffset[1];
      if(ts1>=te0-coinc_dt) break;  // after

      // frequency coincidence
      fs0=triggers[0]->GetClusterFrequencyStart(c0);
      fe0=triggers[0]->GetClusterFrequencyEnd(c0);
      fs1=triggers[1]->GetClusterFrequencyStart(c1);
      fe1=triggers[1]->GetClusterFrequencyEnd(c1);
      if(fs0>fe1+coinc_df) continue;
      if(fs1>fe0+coinc_df) continue;

      // frequency asymmetry coincidence
      f0=triggers[0]->GetClusterFrequency(c0);
      f1=triggers[1]->GetClusterFrequency(c1);
      if(fabs(f1-f0)/(f1+f0)>coinc_af) continue;

      // save coinc
      CoC[0].push_back(c0);
      CoC[1].push_back(c1);
    }
  }
  
  // init coinc tag
  delete CoTag;
  CoTag = new bool [(int)CoC[0].size()];
  for(int c=0; c<(int)CoC[0].size(); c++) CoTag[c]=true;
      
  if(verbosity>1) cout<<"Coinc2::MakeCoinc: "<<CoC[0].size()<<" coinc events found"<<endl;
  return (int)CoC[0].size();
}

////////////////////////////////////////////////////////////////////////////////////
int Coinc2::MakeCoinc0(void){
////////////////////////////////////////////////////////////////////////////////////
  if(triggers[0]==NULL || triggers[1]==NULL){
    cerr<<"Coinc2::MakeCoinc0: both trigger samples must be defined"<<endl;
    return -1;
  }

  // reset coinc
  CoTree->Reset();
  for(int i=0; i<2; i++) CoC[i].clear();
  
  // locals
  double ts0, te0, t1;
  int c1start=0;
  if(verbosity) cout<<"Coinc2::MakeCoinc0: run the 0-type coincidence algo..."<<endl;

  // loop over trigger sample #0
  for(int c0=0; c0<triggers[0]->GetNClusters(); c0++){
    if(triggers[0]->GetClusterTag(c0)==-1) continue;
    triggers[0]->SetClusterTag(c0,1);

    // coinc window = coinc_dt
    ts0=TMath::Max(triggers[0]->GetClusterTime(c0)+toffset[0]-coinc_dt/2.0,triggers[0]->GetClusterTimeStart(c0)+toffset[0]);
    te0=TMath::Min(triggers[0]->GetClusterTime(c0)+toffset[0]+coinc_dt/2.0,triggers[0]->GetClusterTimeEnd(c0)+toffset[0]);
      
    // loop over trigger sample #1
    for(int c1=c1start; c1<triggers[1]->GetNClusters(); c1++){
      if(c1) c1start=c1-1;// assumes clusters with no time overlap
      if(triggers[1]->GetClusterTag(c1)==-1) continue;
      triggers[1]->SetClusterTag(c1,1);
 
      // time coincidence
      t1=triggers[1]->GetClusterTime(c1)+toffset[1];
      if(t1<ts0) continue;// before
      if(t1>te0) break;  // after

      // NO frequency coincidence
      
      // save coinc
      CoC[0].push_back(c0);
      CoC[1].push_back(c1);
    }
  }
  
  // init coinc tag
  delete CoTag;
  CoTag = new bool [(int)CoC[0].size()];
  for(int c=0; c<(int)CoC[0].size(); c++) CoTag[c]=true;
      
  if(verbosity>1) cout<<"Coinc2::MakeCoinc0: "<<CoC[0].size()<<" coinc events found"<<endl;
  return (int)CoC[0].size();
}

////////////////////////////////////////////////////////////////////////////////////
bool Coinc2::WriteCoinc(const string aOutDir, const string aOutFileName){
////////////////////////////////////////////////////////////////////////////////////
  if(triggers[0]==NULL || triggers[1]==NULL){
    cerr<<"Coinc2::WriteCoinc: both trigger samples must be defined"<<endl;
    return false;
  }

  // coinc clusters
  MakeTriggers *CoCl[2];
  CoCl[0] = new MakeTriggers(triggers[0]->GetName(),verbosity);
  CoCl[1] = new MakeTriggers(triggers[1]->GetName(),verbosity);
  CoCl[0]->SetMprocessname("coinc2");
  CoCl[1]->SetMprocessname("coinc2");
  if(!CoCl[0]->GetStatus()||!CoCl[1]->GetStatus()){
    delete CoCl[0];
    delete CoCl[1];
    return false;
  }

  if(verbosity) cout<<"Coinc2::WriteCoinc: fill coinc event structures..."<<endl;
  for(int i=0; i<2; i++){
    for(int c=0; c<(int)CoC[0].size(); c++){
      if(!CoTag[c]) continue;// non-valid coinc
      CoCl[i]->AddTrigger(triggers[i]->GetClusterTime(CoC[i][c]),triggers[i]->GetClusterFrequency(CoC[i][c]),triggers[i]->GetClusterSNR(CoC[i][c]),triggers[i]->GetClusterQ(CoC[i][c]),triggers[i]->GetClusterTimeStart(CoC[i][c]),triggers[i]->GetClusterTimeEnd(CoC[i][c]),triggers[i]->GetClusterFrequencyStart(CoC[i][c]),triggers[i]->GetClusterFrequencyEnd(CoC[i][c]),triggers[i]->GetClusterAmplitude(CoC[i][c]),triggers[i]->GetClusterPhase(CoC[i][c]));
    }
   
    // set coinc segments
    CoCl[i]->Append(CoSeg);
  }

  // write coinc events
  if(verbosity) cout<<"Coinc2::WriteCoinc: write coinc event structures..."<<endl;
  int ncoinc = CoCl[0]->GetNTrig();
  CoCl[0]->Write(aOutDir, "root", aOutFileName,"clusters_0","RECREATE");
  CoCl[1]->Write(aOutDir, "root", aOutFileName,"clusters_1","UPDATE");
  delete CoCl[0];
  delete CoCl[1];

  // save coinc parameters
  TTree *CoParam = new TTree("coinc2param","coinc2param");
  CoParam->Branch("dt", &coinc_dt, "dt/D");
  CoParam->Branch("df", &coinc_df, "df/D");
  CoParam->Branch("af", &coinc_af, "af/D");
  CoParam->Branch("toffset_0", &toffset[0], "toffset_0/D");
  CoParam->Branch("toffset_1", &toffset[1], "toffset_1/D");
  CoParam->Fill();

  // save coinc parameters tree
  if(verbosity) cout<<"Coinc2::WriteCoinc: write coinc2param tree..."<<endl;
  TFile fout((aOutDir+"/"+aOutFileName+".root").c_str(),"UPDATE");
  if(fout.IsZombie()){ delete CoTree; return false; }
  fout.cd();
  CoParam->Write();
  if(CoTree->GetEntries()==ncoinc) CoTree->Write();
  fout.Close();
 
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
void Coinc2::PrintCoinc(void){
////////////////////////////////////////////////////////////////////////////////////
  if(triggers[0]==NULL || triggers[1]==NULL) return;

  // loop over coinc
  for(int c=0; c<(int)CoC[0].size(); c++){
    cout<<"*********************************************************************"<<endl;
    cout<<"coinc_"<<c<<endl;
    cout<<"  "<<triggers[0]->GetStreamName()<<": t="<<triggers[0]->GetClusterTime(CoC[0][c])<<", f="<<triggers[0]->GetClusterFrequency(CoC[0][c])<<", snr="<<triggers[0]->GetClusterSNR(CoC[0][c])<<endl;;
    cout<<"  "<<triggers[1]->GetStreamName()<<": t="<<triggers[1]->GetClusterTime(CoC[1][c])<<", f="<<triggers[1]->GetClusterFrequency(CoC[1][c])<<", snr="<<triggers[1]->GetClusterSNR(CoC[1][c])<<endl;;
  }
  cout<<"*********************************************************************"<<endl;

  return;
}

////////////////////////////////////////////////////////////////////////////////////
void Coinc2::SelectClusters(const int aSampleIndex, const bool aIgnoreMinusOnes){
////////////////////////////////////////////////////////////////////////////////////

  nactive[aSampleIndex]=0;

  // frequency selection is active
  if(freqmin[aSampleIndex]>0||freqmax[aSampleIndex]<1e20){
    for(int c=0; c<triggers[aSampleIndex]->GetNClusters(); c++){
      if(aIgnoreMinusOnes&&triggers[aSampleIndex]->GetClusterTag(c)==-1) continue;
      if(CoSeg->IsInsideSegment(triggers[aSampleIndex]->GetClusterTime(c)+toffset[aSampleIndex])
	 && triggers[aSampleIndex]->GetClusterFrequency(c)>=freqmin[aSampleIndex]
	 && triggers[aSampleIndex]->GetClusterFrequency(c)<freqmax[aSampleIndex]){
	triggers[aSampleIndex]->SetClusterTag(c,1);
	nactive[aSampleIndex]++;
      }
      else triggers[aSampleIndex]->SetClusterTag(c,-1);// discarded
    }
  }
  // frequency selection is NOT active
  else{
    for(int c=0; c<triggers[aSampleIndex]->GetNClusters(); c++){
      if(aIgnoreMinusOnes&&triggers[aSampleIndex]->GetClusterTag(c)==-1) continue;
      if(CoSeg->IsInsideSegment(triggers[aSampleIndex]->GetClusterTime(c)+toffset[aSampleIndex])){ triggers[aSampleIndex]->SetClusterTag(c,1); nactive[aSampleIndex]++; }
      else triggers[aSampleIndex]->SetClusterTag(c,-1);// discarded
    }
  }
    
  if(verbosity)
    cout<<"Coinc2::SelectClusters: trigger sample #"<<aSampleIndex<<" = "<<triggers[aSampleIndex]->GetStreamName()<<", time offset = "<<toffset[aSampleIndex]<<", "<<nactive[aSampleIndex]<<" active clusters"<<endl;

  return;
}
/*
////////////////////////////////////////////////////////////////////////////////////
bool Coinc2::MakeMonitors(const bool aWeighted){
////////////////////////////////////////////////////////////////////////////////////
  if(triggers[0]==NULL || triggers[1]==NULL){
    cerr<<"Coinc2::MakeMonitors: both trigger samples must be defined"<<endl;
    return false;
  }
  if(coinc_dt){
    cerr<<"Coinc2::MakeMonitors: This function is wrong when the coinc dt parameter is non-zero"<<endl;
    return false;
  }

  if(verbosity) cout<<"Coinc2::MakeMonitors: fill monitors..."<<endl;

  TriggerSelect *tselect;
  
  for(int i=0; i<2; i++){

    // input trigger parameter space
    tselect = new TriggerSelect(triggers[i],verbosity);
    tselect->SetFrequencyResolution(TMath::Min(100,triggers[i]->GetNClusters()/2000+3));
    tselect->SetSNRRange(tselect->GetSNRMin(),TMath::Max(100.0,2*tselect->GetSNRMin()));
    tselect->SetSNRResolution(50);
  
    // init monitors
    delete h_ct[i];
    h_ct[i] = tselect->GetTH2D("Frequency","SNR");
    h_ct[i]->SetName(("ct_"+triggers[i]->GetName()).c_str());
    h_ct[i]->SetTitle((triggers[i]->GetStreamName()+": valid cluster time").c_str());
    h_ct[i]->GetXaxis()->SetTitle("Frequency [Hz]");
    h_ct[i]->GetYaxis()->SetTitle("SNR");
    h_ct[i]->GetZaxis()->SetTitle("cluster time [s]");

    delete h_ctcum[i];
    h_ctcum[i] = tselect->GetTH2D("Frequency","SNR");
    h_ctcum[i]->SetName(("ctcum_"+triggers[i]->GetName()).c_str());
    h_ctcum[i]->SetTitle((triggers[i]->GetStreamName()+": cumulative valid cluster time").c_str());
    h_ctcum[i]->GetXaxis()->SetTitle("Frequency [Hz]");
    h_ctcum[i]->GetYaxis()->SetTitle("SNR threshold");
    h_ctcum[i]->GetZaxis()->SetTitle("cluster time [s]");

    delete h_ct_coinc[i];
    h_ct_coinc[i] = tselect->GetTH2D("Frequency","SNR");
    h_ct_coinc[i]->SetName(("ct_coinc_"+triggers[i]->GetName()).c_str());
    h_ct_coinc[i]->SetTitle((triggers[i]->GetStreamName()+": coinc cluster time").c_str());
    h_ct_coinc[i]->GetXaxis()->SetTitle("Frequency [Hz]");
    h_ct_coinc[i]->GetYaxis()->SetTitle("SNR");
    h_ct_coinc[i]->GetZaxis()->SetTitle("cluster time [s]");

    delete h_ctcum_coinc[i];
    h_ctcum_coinc[i] = tselect->GetTH2D("Frequency","SNR");
    h_ctcum_coinc[i]->SetName(("ctcum_coinc_"+triggers[i]->GetName()).c_str());
    h_ctcum_coinc[i]->SetTitle((triggers[i]->GetStreamName()+": cumulative coinc cluster time").c_str());
    h_ctcum_coinc[i]->GetXaxis()->SetTitle("Frequency [Hz]");
    h_ctcum_coinc[i]->GetYaxis()->SetTitle("SNR threshold");
    h_ctcum_coinc[i]->GetZaxis()->SetTitle("cluster time [s]");

    delete tselect;

    // fill cluster monitors
    double ct;
    for(int c=0; c<triggers[i]->GetNClusters(); c++){
      if(triggers[i]->GetClusterTag(c)<0) continue;
      if(!i && smartshift) ct = GetSmartShiftEnd(i,c) - GetSmartShiftStart(i,c);
      else ct=triggers[i]->GetClusterDuration(c);
      h_ct[i]->Fill(triggers[i]->GetClusterFrequency(c),triggers[i]->GetClusterSNR(c),ct);
    }
  }
  
  // fill coinc monitors
  double t0min, t0max, t1min, t1max; 
  for(int co=0; co<(int)CoC[0].size(); co++){
    if(!CoTag[co]) continue;

    // timing
    if(smartshift){
      t0min=GetSmartShiftStart(0,CoC[0][co]);
      t0max=GetSmartShiftEnd(0,CoC[0][co]);
    }
    else{
      t0min=triggers[0]->GetClusterTimeStart(CoC[0][co])+toffset[0];
      t0max=triggers[0]->GetClusterTimeEnd(CoC[0][co])+toffset[0];
    }
    t1min=triggers[1]->GetClusterTimeStart(CoC[1][co])+toffset[1];
    t1max=triggers[1]->GetClusterTimeEnd(CoC[1][co])+toffset[1];

    // fill monitor
    ct=TMath::Min(t0max,t1max)-TMath::Max(t0min,t1min);
    h_ct_coinc[0]->Fill(triggers[0]->GetClusterFrequency(CoC[0][co]),triggers[0]->GetClusterSNR(CoC[0][co]),ct);
    h_ct_coinc[1]->Fill(triggers[1]->GetClusterFrequency(CoC[1][co]),triggers[1]->GetClusterSNR(CoC[1][co]),ct);
  }

  // make cumulative monitors
  for(int i=0; i<2; i++){
    for(int f=0; f<=h_n[i]->GetNbinsX()+1; f++){
      
      // init with SNR overflow
      h_ncum[i]->SetBinContent(f,h_n[i]->GetNbinsY()+1,h_n[i]->GetBinContent(f,h_n[i]->GetNbinsY()+1));
      h_ncum[i]->SetBinError(f,h_n[i]->GetNbinsY()+1,h_n[i]->GetBinError(f,h_n[i]->GetNbinsY()+1));
      h_ncum_coinc[i]->SetBinContent(f,h_n_coinc[i]->GetNbinsY()+1,h_n_coinc[i]->GetBinContent(f,h_n_coinc[i]->GetNbinsY()+1));
      h_ncum_coinc[i]->SetBinError(f,h_n_coinc[i]->GetNbinsY()+1,h_n_coinc[i]->GetBinError(f,h_n_coinc[i]->GetNbinsY()+1));
      
      // go down in SNR
      for(int s=h_n[i]->GetNbinsY(); s>=0; s--){
	h_ncum[i]->SetBinContent(f,s,h_ncum[i]->GetBinContent(f,s+1)+h_n[i]->GetBinContent(f,s));
	h_ncum[i]->SetBinError(f,s,sqrt(h_ncum[i]->GetBinError(f,s+1)*h_ncum[i]->GetBinError(f,s+1)+h_n[i]->GetBinError(f,s)*h_n[i]->GetBinError(f,s)));
	h_ncum_coinc[i]->SetBinContent(f,s,h_ncum_coinc[i]->GetBinContent(f,s+1)+h_n_coinc[i]->GetBinContent(f,s));
	h_ncum_coinc[i]->SetBinError(f,s,sqrt(h_ncum_coinc[i]->GetBinError(f,s+1)*h_ncum_coinc[i]->GetBinError(f,s+1)+h_n_coinc[i]->GetBinError(f,s)*h_n_coinc[i]->GetBinError(f,s)));
      }
    }

  }
  
  return true;
}

////////////////////////////////////////////////////////////////////////////////////
bool Coinc2::WriteMonitors(const string aOutDir, const string aOutFileName){
////////////////////////////////////////////////////////////////////////////////////
  if(verbosity) cout<<"Coinc2::WriteMonitors: save monitors..."<<endl;
  TFile fout((aOutDir+"/"+aOutFileName+".root").c_str(),"RECREATE");
  if(fout.IsZombie()) return false;
  fout.cd();
  for(int i=0; i<2; i++){
    h_n[i]->Write();
    h_ncum[i]->Write();
    h_n_coinc[i]->Write();
    h_ncum_coinc[i]->Write();
  }
  fout.Close();
  return true;
}
*/
////////////////////////////////////////////////////////////////////////////////////
bool Coinc2::MakeComparators(const string aTriggerId0, const string aTriggerId1){
////////////////////////////////////////////////////////////////////////////////////
  if(triggers[0]==NULL || triggers[1]==NULL){
    cerr<<"Coinc2::MakeComparators: both trigger samples must be defined"<<endl;
    return false;
  }

  if(verbosity) cout<<"Coinc2::MakeComparators: fill comparators..."<<endl;

  TriggerSelect *tselect;
 
  // clean previous comparators
  delete hc_asnr;
  delete hc_snrsnr;
  delete hc_af;
  delete hc_ff;

  // input trigger parameter space
  tselect = new TriggerSelect(triggers[0],verbosity);
  tselect->SetFrequencyResolution(TMath::Min(100,triggers[0]->GetNClusters()/1000+3));
  tselect->SetSNRRange(tselect->GetSNRMin(),TMath::Max(100.0,2*tselect->GetSNRMax()));
  tselect->SetSNRResolution(100);

  // init comparators
  hc_snrsnr = tselect->GetTH2D("SNR","SNR");
  hc_snrsnr->SetName("c_snrsnr");
  hc_snrsnr->SetTitle((aTriggerId1+"/"+aTriggerId0+": SNR comparison").c_str());
  hc_snrsnr->GetXaxis()->SetTitle((aTriggerId0+" SNR").c_str());
  hc_snrsnr->GetYaxis()->SetTitle((aTriggerId1+" SNR").c_str());
  hc_snrsnr->GetZaxis()->SetTitle("Number of clusters");

  hc_asnr = new TH2D("c_asnr",(aTriggerId1+"/"+aTriggerId0+": SNR comparison").c_str(),hc_snrsnr->GetNbinsX(),hc_snrsnr->GetXaxis()->GetXbins()->GetArray(),200,-1,1); 
  hc_asnr->GetYaxis()->SetTitle(("SNR asymmetry ("+aTriggerId1+"/"+aTriggerId0+")").c_str());
  hc_asnr->GetXaxis()->SetTitle((aTriggerId0+" SNR").c_str());
  hc_asnr->GetZaxis()->SetTitle("Number of clusters");
    
  hc_ff = tselect->GetTH2D("Frequency","Frequency");
  hc_ff->SetName("c_ff");
  hc_ff->SetTitle((aTriggerId1+"/"+aTriggerId0+": Frequency comparison").c_str());
  hc_ff->GetXaxis()->SetTitle((aTriggerId0+" frequency [Hz]").c_str());
  hc_ff->GetYaxis()->SetTitle((aTriggerId1+" frequency [Hz]").c_str());
  hc_ff->GetZaxis()->SetTitle("Number of clusters");

  hc_af = new TH2D("c_af",(aTriggerId1+"/"+aTriggerId0+": Frequency comparison").c_str(),hc_snrsnr->GetNbinsX(),hc_snrsnr->GetXaxis()->GetXbins()->GetArray(),100,-coinc_af,coinc_af); 
  hc_af->GetXaxis()->SetTitle((aTriggerId0+" SNR").c_str());
  hc_af->GetYaxis()->SetTitle(("Frequency asymmetry ("+aTriggerId1+"/"+aTriggerId0+")").c_str());
  hc_af->GetZaxis()->SetTitle("Number of clusters");
    
  delete tselect;

  // fill comparators
  for(int co=0; co<(int)CoC[0].size(); co++){
    if(!CoTag[co]) continue;// non-valid coinc

    hc_asnr->Fill(triggers[0]->GetClusterSNR(CoC[0][co]),(triggers[1]->GetClusterSNR(CoC[1][co])-triggers[0]->GetClusterSNR(CoC[0][co]))/(triggers[1]->GetClusterSNR(CoC[1][co])+triggers[0]->GetClusterSNR(CoC[0][co])));
    hc_snrsnr->Fill(triggers[0]->GetClusterSNR(CoC[0][co]),triggers[1]->GetClusterSNR(CoC[1][co]));
    hc_af->Fill(triggers[0]->GetClusterSNR(CoC[0][co]),(triggers[1]->GetClusterFrequency(CoC[1][co])-triggers[0]->GetClusterFrequency(CoC[0][co]))/(triggers[1]->GetClusterFrequency(CoC[1][co])+triggers[0]->GetClusterFrequency(CoC[0][co])));
    hc_ff->Fill(triggers[0]->GetClusterFrequency(CoC[0][co]),triggers[1]->GetClusterFrequency(CoC[1][co]));
  }
  
  return true;
}


////////////////////////////////////////////////////////////////////////////////////
bool Coinc2::PrintComparator(const string aComparatorName){
////////////////////////////////////////////////////////////////////////////////////
  
  if(!aComparatorName.compare("hc_asnr")&&hc_asnr->GetEntries()){
    GwollumPlot::Draw(hc_asnr,"COLZ");
  }
  else if(!aComparatorName.compare("hc_snrsnr")&&hc_snrsnr->GetEntries()){
    GwollumPlot::Draw(hc_snrsnr,"COLZ");
  }
  else if(!aComparatorName.compare("hc_af")&&hc_af->GetEntries()){
    GwollumPlot::Draw(hc_af,"COLZ");
  }
  else if(!aComparatorName.compare("hc_ff")&&hc_ff->GetEntries()){
    GwollumPlot::Draw(hc_ff,"COLZ");
  }
  else{
    cerr<<"Coinc2::PrintComparator: Comparator \""<<aComparatorName<<"\" does not exist"<<endl;
    return false;
  }
  return true;
}
