//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __Triggers__
#define __Triggers__

#include "TROOT.h"
#include "TChain.h"
#include "TTreeIndex.h"
#include "Segments.h"

using namespace std;

/**
 * Manage a trigger set.
 * This class is designed to manage a set of triggers. A trigger is described as a set of parameters and these parameters are stored in a ROOT TTree as described in the <a href="../../Main/convention.html#triggers">GWOLLUM convention</a> for triggers.
 *
 * The trigger set can be defined according to two modes and some methods are mode-specific:
 * - the 'reading-mode': the trigger set is loaded from ROOT files
 * - the 'building-mode': the trigger set is built from scratch and triggers are saved in memory.
 *
 * PLEASE NOTE that this class is rather technical and the user is invited to use the MakeTriggers or ReadTriggers classes instead.
 *
 * This class also offers the possibility to cluster the set of triggers with an algorithm called Clusterize. For the description of the clustering algorithms, see [FIXME]
 * \author    Florent Robinet
 */
class Triggers {

 public:

  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the Triggers class.
   * The trigger and cluster trees are initialized. By default, the building-mode is selected.
   *
   * Parameters are set to default:
   * - Clustering delta_t  = 0.1 s (SetClusterizeDt())
   * - Clustering SNR threshold = -1.0 (SetClusterizeSNRthr())
   *
   * @param aVerbose verbosity level
   */
  Triggers(const int aVerbose=0);

  /**
   * Destructor of the Triggers class.
   */
  virtual ~Triggers(void);
  /**
     @}
  */


  /**
   * Adds a trigger to the Triggers object (only available in building-mode).
   * The trigger parameters are defined in the <a href="../../Main/convention.html#triggers">GWOLLUM convention</a> for triggers. The timing parameters (time, tstart and tend) are recorded with a precision of 1 microsecond.
   *
   * @param aTime
   * @param aFrequency
   * @param aSNR
   * @param aQ
   * @param aTstart = Min( atstart, atime )
   * @param aTend = Max( atend, atime )
   * @param aFstart = Min( afstart, afrequency )
   * @param aFend = Max( afend, afrequency )
   * @param aAmplitude
   * @param aPhase
   */
  bool AddTrigger(const double aTime,   const double aFrequency,
		  const double aSNR,    const double aQ,
		  const double aTstart, const double aTend,
		  const double aFstart, const double aFend,
		  const double aAmplitude, const double aPhase);

  /**
   * Sort triggers by increasing values of tstart (only available in building-mode).
   */
  bool SortTriggers(void);

  /**
   * Modifies the branch status of the trigger tree.
   * This function allows to change the status of the trigger tree branches. See also <a href="http://root.cern.ch/root/htmldoc/TTree.html#TTree:SetBranchStatus">TTree::SetBranchStatus( )</a>.
   * This function is particularly useful to speed up processes looping over triggers.
   *
   * IMPORTANT: The user MUST re-activate the branch before using any other class functions.
   * @param aBname branch name
   * @param aStatus new status
   */
  bool SetTriggerBranchStatus(const string aBname, const bool aStatus = true);

  /**
   * Returns the current number of triggers in memory.
   */
  inline int GetNTrig(void){ return Ttree->GetEntries(); };

  /**
   * Returns the current number of triggers in memory.
   */
  inline int GetNtrig(void){ return Ttree->GetEntries(); };

  /**
   * Returns the current number of triggers in memory.
   */
  inline int GetNTriggers(void){ return Ttree->GetEntries(); };

  /**
   * Returns the current number of triggers in memory.
   */
  inline int GetNtriggers(void){ return Ttree->GetEntries(); };

  /**
   * Returns the file path containing the trigger with index aTr (only available in building-mode).
   * "none" is returned if this function fails.
   * @param aTr trigger index
   */
  string GetTriggerFile(const int aTr);

  /**
   * Returns the time of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  inline double GetTriggerTime(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Ttime;
    return 0.0;
  };

  /**
   * Returns the frequency of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  inline double GetTriggerFrequency(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Tfreq;
    return 0.0;
  };

  /**
   * Returns the SNR of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  double GetTriggerSNR(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Tsnr;
    return 0.0;
  };

  /**
   * Returns the amplitude of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  double GetTriggerAmplitude(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Tamp;
    return 0.0;
  };

  /**
   * Returns the Q of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  double GetTriggerQ(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Tq;
    return 0.0;
  };

  /**
   * Returns the phase of trigger with index aTr.
   * -100.0 is returned if this function fails.
   * @param aTr trigger index
   */
  double GetTriggerPhase(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Tph;
    return 0.0;
  };

  /**
   * Returns the starting time of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  double GetTriggerTimeStart(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Ttstart;
    return 0.0;
  };

  /**
   * Returns the ending time of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  double GetTriggerTimeEnd(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Ttend;
    return 0.0;
  };

  /**
   * Returns the duration of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  inline double GetTriggerDuration(const int aTr){
    return GetTriggerTimeEnd(aTr)-GetTriggerTimeStart(aTr);
  };

  /**
   * Returns the starting frequency of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  double GetTriggerFrequencyStart(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Tfstart;
    return 0.0;
  };

  /**
   * Returns the ending frequency of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  double GetTriggerFrequencyEnd(const int aTr){
    if(Ttree->GetEntry(aTr)>0) return Tfend;
    return 0.0;
  };

  /**
   * Returns the bandwidth of trigger with index aTr.
   * 0.0 is returned if this function fails.
   * @param aTr trigger index
   */
  double GetTriggerBandWidth(const int aTr){
    return GetTriggerFrequencyEnd(aTr)-GetTriggerFrequencyStart(aTr);
  };

  /**
   * Returns the index of trigger overlapping a given GPS time.
   * -1 is returned if no trigger is found.
   * @param aTime GPS time to test
   */
  int GetTriggerIndex(const double aTime);

  /**
   * Returns the time segments associated to a trigger selection.
   * For each trigger, the segment [tstart; tend[ is added to a list of segments. The final list is returned as a Segments pointer.
   *
   * It is possible to perform a trigger selection where only triggers above a given threshold are used. The TH1D threshold object is used as a SNR threshold defined in frequency bins. Additionnaly, if the SNR threshold in a given bin is larger than 'aInfValue', the threshold is considered infinite and no trigger can be selected for this bin.
   *
   * NOTE: for performance reasons, it is better to sort the triggers before calling this function(see SortTriggers()).
   * @param aThr threshold object
   * @param aInfValue value above which the threshold is considered infinite
   */
  Segments* GetTriggerSegments(TH1D *aThr=NULL, const double aInfValue=1e20);

  /**
   * Resets the clusters.
   * The clusters are flushed out of memory.
   */
  void ResetClusters(void);
  
  /**
   * Applies the so-called Clusterize algorithm to the trigger set.
   * If the triggers were previously clustered, the memory is cleaned first (see ResetClusters()).
   *
   * The Clusterize algorithm assumes the trigger set is sorted by increasing values of tstart. The user must make sure this is the case. The SortTriggers() function can be used to sort the triggers before the clustering.
   *
   * Resulting clusters are tagged. By default, this tag is -1 for all clusters.
   *
   * The clustering algorithms are described in FIXME
   * @param aTag default tag for all clusters
   */
  bool Clusterize(const int aTag=-1);

  /**
   * Sets a new clustering delta_t parameter.
   * Sets a new delta_t value for the time-clustering. The value should be given in seconds.
   * @param aDt new delta_t value in seconds
   */
  inline void SetClusterizeDt(const double aDt){Cdelta_t=aDt; return;};

  /**
   * Sets a minimal size for clustering.
   * Only clusters with a size greater than this threshold will be recorded.
   * @param aSizeMin new size threshold
   */
  inline void SetClusterizeSizeMin(const double aSizeMin){Csizemin=aSizeMin; return;};

  /**
   * Sets a SNR threshold for clustering. 
   * Only clusters with a SNR greater than this threshold will be recorded.
   * @param aSNRthr new SNR threshold
   */
  inline void SetClusterizeSNRthr(const double aSNRthr){cl_snr_thr=aSNRthr; return;};

  /**
   * Sets a new cluster tag value.
   * This function allows to assign a new tag value to cluster 'aCl'.
   * @param aCl cluster index
   * @param aNewTag new cluster tag value
   */
  bool SetClusterTag(const int aCl, const int aNewTag);

  /**
   * Sets a new cluster tag value for all cluster.
   * This function allows to assign a new tag value all clusters.
   * @param aNewTag new cluster tag value
   */
  inline void SetClusterTag(const int aNewTag){
    for(int c=0; c<Ctree->GetEntries(); c++) cl_tag[c]=aNewTag;
  };

  /**
   * Modifies the branch status of the cluster tree.
   * This function allows to change the status of the cluster tree branches. See also <a href="http://root.cern.ch/root/htmldoc/TTree.html#TTree:SetBranchStatus">TTree::SetBranchStatus( )</a>.
   * This function is particularly useful to speed up processes looping over clusters. 
   *
   * IMPORTANT: The user MUST re-activate the branch before using any other class functions.
   * @param aBname branch name
   * @param aStatus new status
   */
  bool SetClusterBranchStatus(const string aBname, const bool aStatus = true);

  /**
   * Returns the number of clusters.
   */
  inline int GetNClust(void){return Ctree->GetEntries();};
  /**
   * Returns the number of clusters.
   */
  inline int GetNclust(void){return Ctree->GetEntries();};

  /**
   * Returns the number of clusters.
   */
  inline int GetNClusters(void){return Ctree->GetEntries();};
  /**
   * Returns the number of clusters.
   */
  inline int GetNclusters(void){return Ctree->GetEntries();};

  /**
   * Returns the clustering delta_t parameter.
   */
  inline double GetClusterizeDt(void){return Cdelta_t;};
  
  /**
   * Returns the time of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterTime(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Ctime;
    return 0.0;
  };

  /**
   * Returns the frequency of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterFrequency(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Cfreq;
    return 0.0;
  };

  /**
   * Returns the SNR of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterSNR(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Csnr;
    return 0.0;
  };

  /**
   * Returns the amplitude of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterAmplitude(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Camp;
    return 0.0;
  };

  /**
   * Returns the Q of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterQ(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Cq;
    return 0.0;
  };

  /**
   * Returns the phase of cluster with index aCl.
   * -100.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterPhase(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Cph;
    return -100.0;
  };

  /**
   * Returns the starting time of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterTimeStart(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Ctstart;
    return 0.0;
  };

  /**
   * Returns the ending time of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterTimeEnd(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Ctend;
    return 0.0;
  };

  /**
   * Returns the duration of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterDuration(const int aCl){
    return GetClusterTimeEnd(aCl)-GetClusterTimeStart(aCl);
  };

  /**
   * Returns the starting frequency of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterFrequencyStart(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Cfstart;
    return 0.0;
  };

  /**
   * Returns the ending frequency of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterFrequencyEnd(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return Cfend;
    return 0.0;
  };

  /**
   * Returns the bandwidth of cluster with index aCl.
   * 0.0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline double GetClusterBandWidth(const int aCl){
    return GetClusterFrequencyEnd(aCl)-GetClusterFrequencyStart(aCl);
  };

  /**
   * Returns the size of cluster with index aCl.
   * 0 is returned if this function fails.
   * @param aCl cluster index
   */
  inline int GetClusterSize(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return (int)Csize;
    return 0;
  };

  /**
   * Returns the index of the first trigger in this cluster.
   * -1 is returned if this function fails.
   * @param aCl cluster index
   */
  inline int GetClusterFirstTrigger(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return (int)Cfirstentry;
    return -1;
  };

  /**
   * Returns the index of cluster overlapping a given GPS time.
   * -1 is returned if no cluster is found.
   * @param aTime GPS time to test
   */
  int GetClusterIndex(const double aTime);

  /**
   * Returns the tag value of cluster with index aCl.
   * -100 is returned if no cluster is found.
   * @param aCl cluster index
   */
  inline int GetClusterTag(const int aCl){
    if(Ctree->GetEntry(aCl)>0) return cl_tag[aCl];
    return -100;
  }

  /**
   * Returns the mean duration of clusters [s].
   */
  inline double GetClusterMeanDuration(void){ return Cmeandur; };
  
  /**
   * Returns the maximal duration of clusters [s].
   */
  inline double GetClusterMaxDuration(void){ return Cmaxdur; };
  
  /**
   * Returns the total duration of clusters [s].
   */
  inline double GetClusterTotalDuration(void){ return Ctotdur; };
  
  /**
   * Resets the triggers.
   * All triggers and clusters are flushed out. The reading mode is removed.
   */
  void ResetTriggers(void);
  
  /**
   * Reads a set of trigger files.
   * As soon as this function is called, the Triggers object is switched to a reading mode. Some functions are therefore de-activated. The reading mode can be removed with the ResetTriggers() function.
   * This function can be used multiple times in a row to update the trigger set.
   * @param aPattern trigger file pattern
   * @param aDirectory ROOT directory where to find the 'triggers' tree.
   */
  bool ReadTriggerFiles(const string aPattern, const string aDirectory);

 protected:

  // GENERAL
  int Verbose;             ///< verbosity level

  // TRIGGERS
  TTree *Ttree;            ///< pointer to active Ttree
  TChain *Ttree_read;      ///< trigger parameter trees reading-mode
  TTree *Ttree_make;       ///< trigger parameter tree building-mode
  double Ttime,            ///< trigger time (TTree)
    Tfreq,                 ///< trigger frequency (TTree)
    Tq,                    ///< trigger Q (TTree)
    Tsnr,                  ///< trigger SNR (TTree)
    Ttstart,               ///< trigger starting time (TTree)
    Ttend,                 ///< trigger ending time (TTree)
    Tfstart,               ///< trigger starting frequency (TTree)
    Tfend,                 ///< trigger ending frequency (TTree)
    Tamp,                  ///< trigger amplitude (TTree)
    Tph;                   ///< trigger phase (TTree)

  // CLUSTERS
  TTree *Ctree;            ///< cluster parameter tree
  double Ctime,            ///< cluster time (TTree)
    Ctstart,               ///< cluster time start (TTree)
    Ctend,                 ///< cluster time end (TTree)
    Cfreq,                 ///< cluster frequency (TTree)
    Cfstart,               ///< cluster frequency start (TTree)
    Cfend,                 ///< cluster frequency end (TTree)
    Csnr,                  ///< cluster SNR (TTree)
    Camp,                  ///< cluster Amplitude (TTree)
    Cq,                    ///< cluster Q (TTree)
    Cph;                   ///< cluster phase (TTree)
  Long64_t Cfirstentry,    ///< cluster first trigger entry (TTree)
    Csize;                 ///< cluster size (TTree)
  int *cl_tag;             ///< cluster tag
  double Cmeandur;         ///< cluster mean duration
  double Cmaxdur;          ///< cluster max duration
  double Ctotdur;          ///< cluster total duration

  // READING-MODE ONLY
  Long64_t *Toffsets;      ///< tree offsets in the chain - DO NOT DELETE

  // BUILDING-MODE ONLY
  int Ttstart_ms;          ///< trigger starting time (ms) (TTree)
  
 private:

  // GENERAL
  bool readmode;           ///< flags reading-only mode
  int randid;              ///< random integer
  string srandid;          ///< random integer (string)

  // CLUSTERIZE
  double Cdelta_t;         ///< time clustering delta_t (s)
  int Csizemin;            ///< size min for clustering
  double cl_snr_thr;       ///< SNR threshold for clustering
  bool Clusterize_t(void); ///< clustering algo
  
  ClassDef(Triggers,0)  
};

#endif


